require 'json'

module UU
  module OS
    module VUC

      # Form event object.
      class Event

        # Raw event attributes.
        # @private
        attr_reader :raw_attrs

        # Unique event ID.
        # @return [String]
        attr_reader :id

        # Event code.
        # @return [String]
        attr_reader :code

        # ID of component which invoked event.
        # @return [String]
        attr_reader :source_id

        # URI of object which invoked event (e.g. process URI).
        # @return [String]
        attr_reader :source_uri

        # ID of form which event belongs to.
        # @return [String]
        attr_reader :form

        # Creates new instance of event.
        #
        # @param [String, Hash] data Initialization data in form of JSON string or Hash (optional)
        def initialize(data = nil)
          if data.nil?
            @raw_attrs = {}
            return
          end
          if data.kind_of?String
            data = JSON.parse(data, :symbolize_names => true)
          end
          if data.kind_of?Hash
            @raw_attrs = data
          elsif data.kind_of?Event
            if data.raw_attrs
              @raw_attrs = data.raw_attrs
            else
              @raw_attrs = {}
            end
          else
            raise ArgumentError.new("Event data must be JSON String or Hash, but was #{data.class}.")
          end
        end

        def raw_attrs
          # Disallow manipulating with raw attributes
          @raw_attrs.clone if @raw_attrs
        end

        def id
          @raw_attrs[:id]
        end

        def code
          @raw_attrs[:code]
        end

        def source_id
          @raw_attrs[:sourceId]
        end

        def source_uri
          @raw_attrs[:sourceUri]
        end

        def form
          @raw_attrs[:form]
        end

      end

    end
  end
end
